@@ -26,12 +26,10 @@
 #ifndef ROCWMMA_CONFIG_HPP
 #define ROCWMMA_CONFIG_HPP
 
-namespace rocwmma
-{
-
 ///
 /// Architecture support
 /// Guaranteed symbols:
+/// ROCWMMA_DEVICE_COMPILE
 /// ROCWMMA_ARCH_GFX908
 /// ROCWMMA_ARCH_GFX90a
 /// ROCWMMA_ARCH_GFX940
@@ -40,24 +38,42 @@ namespace rocwmma
 /// ROCWMMA_ARCH_GFX1100
 /// ROCWMMA_ARCH_GFX1101
 /// ROCWMMA_ARCH_GFX1102
-#if defined(__gfx908__)
-#define ROCWMMA_ARCH_GFX908 __gfx908__
-#elif defined(__gfx90a__)
-#define ROCWMMA_ARCH_GFX90A __gfx90a__
-#elif defined(__gfx940__)
-#define ROCWMMA_ARCH_GFX940 __gfx940__
-#elif defined(__gfx941__)
-#define ROCWMMA_ARCH_GFX941 __gfx941__
-#elif defined(__gfx942__)
-#define ROCWMMA_ARCH_GFX942 __gfx942__
-#elif defined(__gfx1100__)
-#define ROCWMMA_ARCH_GFX1100 __gfx1100__
-#elif defined(__gfx1101__)
-#define ROCWMMA_ARCH_GFX1101 __gfx1101__
-#elif defined(__gfx1102__)
-#define ROCWMMA_ARCH_GFX1102 __gfx1102__
+/// ROCWMMA_ARCH_GFX1200
+/// ROCWMMA_ARCH_GFX1201
+///
+/// IMPORTANT: __gfx908__ and similar macros are exclusively defined during the device
+///            compiler pass, and all other macros rely on their definition.
+///
+///            Device compiler pass: https://rocm.docs.amd.com/projects/HIP/en/latest/user_guide/hip_porting_guide.html#identifying-current-compilation-pass-host-or-device
+#if defined(__HIP_DEVICE_COMPILE__) && __HIP_DEVICE_COMPILE__
+#define ROCWMMA_DEVICE_COMPILE 1
 #else
+#define ROCWMMA_DEVICE_COMPILE 0
+#endif
+#if defined(__gfx908__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX908 __gfx908__
+#elif defined(__gfx90a__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX90A __gfx90a__
+#elif defined(__gfx940__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX940 __gfx940__
+#elif defined(__gfx941__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX941 __gfx941__
+#elif defined(__gfx942__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX942 __gfx942__
+#elif defined(__gfx1100__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX1100 __gfx1100__
+#elif defined(__gfx1101__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX1101 __gfx1101__
+#elif defined(__gfx1102__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX1102 __gfx1102__
+#elif defined(__gfx1200__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX1200 __gfx1200__
+#elif defined(__gfx1201__) && ROCWMMA_DEVICE_COMPILE
+#define ROCWMMA_ARCH_GFX1201 __gfx1201__
+#elif !ROCWMMA_DEVICE_COMPILE
 #define ROCWMMA_ARCH_HOST 1
+#else
+static_assert(0, "Unsupported architecture");
 #endif
 
 #if !defined(ROCWMMA_ARCH_GFX908)
@@ -84,6 +100,12 @@ namespace rocwmma
 #if !defined(ROCWMMA_ARCH_GFX1102)
 #define ROCWMMA_ARCH_GFX1102 0
 #endif
+#if !defined(ROCWMMA_ARCH_GFX1200)
+#define ROCWMMA_ARCH_GFX1200 0
+#endif
+#if !defined(ROCWMMA_ARCH_GFX1201)
+#define ROCWMMA_ARCH_GFX1201 0
+#endif
 #if !defined(ROCWMMA_ARCH_HOST)
 #define ROCWMMA_ARCH_HOST 0
 #endif
@@ -93,10 +115,12 @@ namespace rocwmma
 /// Guaranteed symbols:
 /// ROCWMMA_ARCH_GFX9
 /// ROCWMMA_ARCH_GFX11
+/// ROCWMMA_ARCH_GFX12
 /// ROCWMMA_WAVE64_MODE
 /// ROCWMMA_WAVE32_MODE
 /// ROCWMMA_BLOCK_DIM_16_SUPPORTED
 /// ROCWMMA_BLOCK_DIM_32_SUPPORTED
+/// ROCWMMA_ARCH_GFX94X
 ///
 #if ROCWMMA_ARCH_GFX908 || ROCWMMA_ARCH_GFX90A || ROCWMMA_ARCH_GFX940 || ROCWMMA_ARCH_GFX941 \
     || ROCWMMA_ARCH_GFX942
@@ -112,12 +136,25 @@ namespace rocwmma
 #define ROCWMMA_BLOCK_DIM_16_SUPPORTED 1
 #endif
 
+#if ROCWMMA_ARCH_GFX1200 || ROCWMMA_ARCH_GFX1201
+#define ROCWMMA_ARCH_GFX12 1
+#define ROCWMMA_WAVE32_MODE 1
+#define ROCWMMA_BLOCK_DIM_16_SUPPORTED 1
+#endif
+
+#if ROCWMMA_ARCH_GFX940 || ROCWMMA_ARCH_GFX941 || ROCWMMA_ARCH_GFX942
+#define ROCWMMA_ARCH_GFX94X 1
+#endif
+
 #if !defined(ROCWMMA_ARCH_GFX9)
 #define ROCWMMA_ARCH_GFX9 0
 #endif
 #if !defined(ROCWMMA_ARCH_GFX11)
 #define ROCWMMA_ARCH_GFX11 0
 #endif
+#if !defined(ROCWMMA_ARCH_GFX12)
+#define ROCWMMA_ARCH_GFX12 0
+#endif
 #if !defined(ROCWMMA_WAVE64_MODE)
 #define ROCWMMA_WAVE64_MODE 0
 #endif
@@ -130,6 +167,9 @@ namespace rocwmma
 #if !defined(ROCWMMA_BLOCK_DIM_32_SUPPORTED)
 #define ROCWMMA_BLOCK_DIM_32_SUPPORTED 0
 #endif
+#if !defined(ROCWMMA_ARCH_GFX94X)
+#define ROCWMMA_ARCH_GFX94X 0
+#endif
 
 #if defined(NDEBUG)
 #define ROCWMMA_UNSUPPORTED_IMPL(MSG)
@@ -143,27 +183,28 @@ namespace rocwmma
 #define ROCWMMA_NO_HALF 0
 #endif // HIP_NO_HALF
 
-#if ROCWMMA_NO_HALF || (!ROCWMMA_NO_HALF && defined(__HIP_NO_HALF_CONVERSIONS__))
-#define ROCWMMA_TESTS_NO_HALF 1
-#else
-#define ROCWMMA_TESTS_NO_HALF 0
-#endif // !ROCWMMA_NO_HALF && defined(__HIP_NO_HALF_CONVERSIONS__)
-
 ///
 /// Sanity checks
 ///
-#if ROCWMMA_ARCH_GFX11
-    static_assert((bool)(ROCWMMA_WAVE32_MODE) && !(bool)(ROCWMMA_WAVE64_MODE),
-                  "rocWMMA supports only wave32 for gfx11 arch");
-    static_assert((bool)(ROCWMMA_BLOCK_DIM_16_SUPPORTED) && !(bool)(ROCWMMA_BLOCK_DIM_32_SUPPORTED),
-                  "rocWMMA supports only block size of 16 for gfx11 arch");
+#if ROCWMMA_ARCH_GFX9
+static_assert(!(bool)(ROCWMMA_WAVE32_MODE) && (bool)(ROCWMMA_WAVE64_MODE),
+              "rocWMMA supports only wave64 for gfx9 arch");
+static_assert((bool)(ROCWMMA_BLOCK_DIM_16_SUPPORTED) && (bool)(ROCWMMA_BLOCK_DIM_32_SUPPORTED),
+              "rocWMMA requires block size of 16 and 32 for gfx9 arch");
 #endif
 
-#if ROCWMMA_ARCH_GFX9
-    static_assert(!(bool)(ROCWMMA_WAVE32_MODE) && (bool)(ROCWMMA_WAVE64_MODE),
-                  "rocWMMA supports only wave64 for gfx9 arch");
-    static_assert((bool)(ROCWMMA_BLOCK_DIM_16_SUPPORTED) && (bool)(ROCWMMA_BLOCK_DIM_32_SUPPORTED),
-                  "rocWMMA requires block size of 16 and 32 for gfx9 arch");
+#if ROCWMMA_ARCH_GFX11
+static_assert((bool)(ROCWMMA_WAVE32_MODE) && !(bool)(ROCWMMA_WAVE64_MODE),
+              "rocWMMA supports only wave32 for gfx11 arch");
+static_assert((bool)(ROCWMMA_BLOCK_DIM_16_SUPPORTED) && !(bool)(ROCWMMA_BLOCK_DIM_32_SUPPORTED),
+              "rocWMMA supports only block size of 16 for gfx11 arch");
+#endif
+
+#if ROCWMMA_ARCH_GFX12
+    static_assert((bool)(ROCWMMA_WAVE32_MODE) && !(bool)(ROCWMMA_WAVE64_MODE),
+                  "rocWMMA supports only wave32 for gfx12 arch");
+    static_assert((bool)(ROCWMMA_BLOCK_DIM_16_SUPPORTED) && !(bool)(ROCWMMA_BLOCK_DIM_32_SUPPORTED),
+                  "rocWMMA supports only block size of 16 for gfx12 arch");
 #endif
 
 ///
@@ -177,6 +218,4 @@ namespace rocwmma
 
 #define ROCWMMA_KERNEL __global__
 
-} // namespace rocwmma
-
 #endif // ROCWMMA_CONFIG_HPP
