@@ -64,24 +64,21 @@ namespace rocwmma
 
         // Outer loop = index 0,
         // Inner loop = index N-1
-        template <std::size_t Depth = 0,
-                  typename Iterator,
-                  typename StrideSpace,
-                  typename Strides2d>
+        template <size_t Depth = 0, typename Iterator, typename StrideSpace, typename Strides2d>
         ROCWMMA_DEVICE static inline auto unroll_right(DataT*        dataPtr,
                                                        Iterator&     in,
                                                        uint32_t      ldm,
                                                        StrideSpace&& strideCounts,
                                                        Strides2d&&   strides2d)
         {
-            static_assert(VecTraits<std::decay_t<StrideSpace>>::size()
-                              == VecTraits<std::decay_t<Strides2d>>::size(),
+            static_assert(VecTraits<decay_t<StrideSpace>>::size()
+                              == VecTraits<decay_t<Strides2d>>::size(),
                           "Mismatched size");
-            auto strideOffset = DataLayout::fromMatrixCoord(std::get<Depth>(strides2d), ldm);
-            auto strideCount  = std::get<Depth>(strideCounts);
+            auto strideOffset = DataLayout::fromMatrixCoord(get<Depth>(strides2d), ldm);
+            auto strideCount  = get<Depth>(strideCounts);
 
             // Last depth layer will invoke the load
-            if constexpr(Depth == (VecTraits<std::decay_t<StrideSpace>>::size() - 1u))
+            if constexpr(Depth == (VecTraits<decay_t<StrideSpace>>::size() - 1u))
             {
 #pragma unroll
                 for(int i = 0; i < strideCount; i++)
@@ -94,7 +91,6 @@ namespace rocwmma
             // Recurse to the next nested layer
             else
             {
-#pragma unroll
                 for(int i = 0; i < strideCount; i++)
                 {
                     unroll_right<Depth + 1>(dataPtr, in, ldm, strideCounts, strides2d);
@@ -136,7 +132,7 @@ namespace rocwmma
             }
 
             // Split the reduced stride space.
-            auto workItemsPerWave = std::max(totalWorkItems / maxWaves, 1u);
+            auto workItemsPerWave = max(totalWorkItems / maxWaves, 1u);
             auto strideSpaceS     = inflate_coord_left(workItemsPerWave - 1u, strideSpaceR) + 1u;
 
             // Add back in the VW dimension, for the full stride
@@ -190,7 +186,7 @@ namespace rocwmma
             }
 
             // Split the reduced stride space.
-            constexpr auto workItemsPerWave = std::max(totalWorkItems / maxWaves, 1u);
+            constexpr auto workItemsPerWave = max(totalWorkItems / maxWaves, 1u);
             constexpr auto strideSpaceS
                 = inflate_coord_left(workItemsPerWave - 1u, strideSpaceR) + 1u;
 
