@@ -73,7 +73,7 @@ namespace rocwmma
 
         using StoreVecTraits = VecTraits<typename Traits::StoreT>;
 
-        template <std::size_t Depth = 0,
+        template <size_t Depth = 0,
                   typename Iterator,
                   typename StrideCounts,
                   typename Strides2d>
@@ -87,7 +87,7 @@ namespace rocwmma
             auto strideCount  = get<Depth>(strideCounts);
 
             // Last depth layer will invoke the load
-            if constexpr(Depth == (VecTraits<std::decay_t<StrideCounts>>::size() - 1u))
+            if constexpr(Depth == (VecTraits<decay_t<StrideCounts>>::size() - 1u))
             {
 #pragma unroll
                 for(int i = 0; i < strideCount; i++)
