@@ -63,30 +63,56 @@ namespace rocwmma
         {
             // bpermute: for the current thread, read from laneId
             template <class BPermuteCtrl>
-            struct amdgcn_ds_bpermute
+            struct amdgcn_ds_bpermute : BPermuteCtrl
             {
-                template <typename InputT>
-                ROCWMMA_DEVICE static inline InputT exec(InputT input, uint32_t laneId)
+                // Feed the current lane id to the threadId modifier
+                template <typename DataT>
+                ROCWMMA_DEVICE static inline DataT exec(DataT input)
                 {
-                    // NOTE: final address is laneId * 4
-                    reinterpret_cast<uint32_t&>(input)
-                        = __builtin_amdgcn_ds_bpermute((BPermuteCtrl::threadCtrl(laneId) << 2),
-                                                       reinterpret_cast<uint32_t const&>(input));
+                    static_assert(sizeof(DataT) == sizeof(uint32_t), "Inputs must be 32 bit");
+
+                    reinterpret_cast<uint32_t&>(input) = __builtin_amdgcn_ds_bpermute(
+                        BPermuteCtrl::threadCtrl(detail::WaveSpace<>::localLaneId()),
+                        reinterpret_cast<uint32_t const&>(input));
+                    return input;
+                }
+
+                // This function takes an override to support pre-calculated threadCtrl
+                template <typename DataT>
+                ROCWMMA_DEVICE static inline DataT exec(DataT input, uint32_t threadCtrl)
+                {
+                    static_assert(sizeof(DataT) == sizeof(uint32_t), "Inputs must be 32 bit");
+
+                    reinterpret_cast<uint32_t&>(input) = __builtin_amdgcn_ds_bpermute(
+                        threadCtrl, reinterpret_cast<uint32_t const&>(input));
                     return input;
                 }
             };
 
             // permute: for the current thread, push my value to laneId
             template <class PermuteCtrl>
-            struct amdgcn_ds_permute
+            struct amdgcn_ds_permute : PermuteCtrl
             {
-                template <typename InputT>
-                ROCWMMA_DEVICE static inline InputT exec(InputT input, uint32_t laneId)
+                // Feed the current lane id to the threadId modifier
+                template <typename DataT>
+                ROCWMMA_DEVICE static inline DataT exec(DataT input)
                 {
-                    // NOTE: final address is laneId * 4
-                    reinterpret_cast<uint32_t&>(input)
-                        = __builtin_amdgcn_ds_permute((PermuteCtrl::threadCtrl(laneId) << 2),
-                                                      reinterpret_cast<uint32_t const&>(input));
+                    static_assert(sizeof(DataT) == sizeof(uint32_t), "Inputs must be 32 bit");
+
+                    reinterpret_cast<uint32_t&>(input) = __builtin_amdgcn_ds_permute(
+                        PermuteCtrl::threadCtrl(detail::WaveSpace<>::localLaneId()),
+                        reinterpret_cast<uint32_t const&>(input));
+                    return input;
+                }
+
+                // This function takes an override to support pre-calculated threadCtrl
+                template <typename DataT>
+                ROCWMMA_DEVICE static inline DataT exec(DataT input, uint32_t threadCtrl)
+                {
+                    static_assert(sizeof(DataT) == sizeof(uint32_t), "Inputs must be 32 bit");
+
+                    reinterpret_cast<uint32_t&>(input) = __builtin_amdgcn_ds_permute(
+                        threadCtrl, reinterpret_cast<uint32_t const&>(input));
                     return input;
                 }
             };
@@ -95,10 +121,22 @@ namespace rocwmma
 
         namespace Ctrl
         {
-            template <uint32_t BlockIdx, uint32_t BlockSize>
-            struct BPermuteBlockBCast
+            struct CtrlBase
             {
+            protected:
+                ROCWMMA_DEVICE static inline uint32_t threadCtrl(uint32_t threadId)
+                {
+                    // NOTE: final address is laneId * 4
+                    return threadId << 2;
+                }
+            };
+
+            template <uint32_t BlockIdx, uint32_t BlockSize>
+            struct BPermuteBlockBCast : CtrlBase
+            {
+
             private:
+                using Base = CtrlBase;
                 enum Traits : uint32_t
                 {
                     BLOCK_OFFSET = BlockIdx * BlockSize,
@@ -110,14 +148,15 @@ namespace rocwmma
                 {
                     // Make sure that the threadId is within range
                     auto tIdx = threadId % BlockSize;
-                    return Traits::BLOCK_OFFSET + tIdx;
+                    return Base::threadCtrl(Traits::BLOCK_OFFSET + tIdx);
                 }
             };
 
             template <uint32_t BlockSize, uint32_t VW, uint32_t ElementShift>
-            struct Interleave
+            struct Interleave : CtrlBase
             {
             private:
+                using Base = CtrlBase;
                 enum Traits : uint32_t
                 {
                     MASK_0 = LsbMask<Log2<BlockSize>::value>::value,
@@ -139,14 +178,15 @@ namespace rocwmma
                           & Traits::MASK_0;
                     const uint32_t offset1 = (threadId >> Log2<VW>::value) & Traits::MASK_1;
                     const uint32_t offset2 = threadId & Traits::MASK_2;
-                    return offset0 + offset1 + offset2;
+                    return Base::threadCtrl(offset0 + offset1 + offset2);
                 }
             };
 
             template <uint32_t GroupSize, uint32_t Distance>
-            struct Rotate
+            struct Rotate : CtrlBase
             {
             private:
+                using Base = CtrlBase;
                 enum Traits : uint32_t
                 {
                     MASK_0 = LsbMask<Log2<GroupSize>::value>::value,
@@ -162,40 +202,12 @@ namespace rocwmma
                     // const uint32_t offset1 = (threadId / GroupSize) * GroupSize
                     const uint32_t offset0 = (threadId + Distance) & Traits::MASK_0;
                     const uint32_t offset1 = threadId & Traits::MASK_1;
-                    return offset0 + offset1;
-                }
-            };
-
-            template <uint32_t GroupSize, uint32_t BlockSize, uint32_t DupCount, uint32_t Offset>
-            struct DuplicateBlocks
-            {
-            private:
-                enum Traits : uint32_t
-                {
-                    MASK_0 = LsbMask<Log2<BlockSize>::value>::value,
-                    MASK_1 = LsbMask<Log2<GroupSize>::value - Log2<DupCount>::value>::value,
-                    MASK_2 = ~LsbMask<Log2<GroupSize>::value>::value,
-                };
-
-            public:
-                // Calculate the read element based on thread position.
-                ROCWMMA_HOST_DEVICE static inline uint32_t threadCtrl(uint32_t threadId)
-                {
-                    // For reference in readibility:
-                    // const uint32_t offset0 = (threadId % BlockSize)
-                    // const uint32_t offset1 = ((threadId / DupCount / BlockSize ) * BlockSize) % (GroupSize / DupCount)
-                    // const uinti32_t offset2 = (threadId / GroupSize) * GroupSize
-                    // constexpr uint32_t offset3 = Offset * BlockSize
-                    const uint32_t offset0 = (threadId & Traits::MASK_0);
-                    const uint32_t offset1
-                        = (threadId >> Log2<DupCount>::value) & ~(~Traits::MASK_0 ^ Traits::MASK_1);
-                    const uint32_t     offset2 = (threadId & Traits::MASK_2);
-                    constexpr uint32_t offset3 = Offset << Log2<BlockSize>::value;
-                    return offset0 + offset1 + offset2 + offset3;
+                    return CtrlBase::threadCtrl(offset0 + offset1);
                 }
             };
 
         } // namespace Ctrl
+
         namespace OpsBase
         {
             template <uint32_t OpId, uint32_t SubGroupSize>
@@ -324,13 +336,13 @@ namespace rocwmma
             using Gather16 = OpsBase::Gather<OP_GROUP_SIZE_16, VW, ElementShift>;
 
             template<uint32_t VW, uint32_t ElementShift>
-            using ScatterWave = OpsBase::Gather<OP_GROUP_SIZE_WARP, VW, ElementShift>;
+            using ScatterWave = OpsBase::Scatter<OP_GROUP_SIZE_WARP, VW, ElementShift>;
 
             template<uint32_t VW, uint32_t ElementShift>
-            using Scatter32 = OpsBase::Gather<OP_GROUP_SIZE_32, VW, ElementShift>;
+            using Scatter32 = OpsBase::Scatter<OP_GROUP_SIZE_32, VW, ElementShift>;
 
             template<uint32_t VW, uint32_t ElementShift>
-            using Scatter16 = OpsBase::Gather<OP_GROUP_SIZE_16, VW, ElementShift>;
+            using Scatter16 = OpsBase::Scatter<OP_GROUP_SIZE_16, VW, ElementShift>;
 
 
             template<uint32_t Distance>
@@ -339,16 +351,6 @@ namespace rocwmma
             template<uint32_t Distance>
             using RotateWaveR = OpsBase::RotateR<Distance, OP_GROUP_SIZE_WARP>;
 
-
-            // template<uint32_t BlockSize>
-            // struct DupLoBlockWave : OpBase<Properties::OP_ID_SHUFFLE, Properties::OP_GROUP_SIZE_WARP, OP_IMPL_BPERM, OP_CTRL>, detail::amdgcn_duplicate_blocks<Properties::OP_GROUP_SIZE_WARP, BlockSize, 2u, 0u>{};
-
-            // template<uint32_t BlockSize>
-            // struct DupHiBlockWave : OpBase<Properties::OP_ID_SHUFFLE, Properties::OP_GROUP_SIZE_WARP, OP_IMPL_BPERM, OP_CTRL>, detail::amdgcn_duplicate_blocks<Properties::OP_GROUP_SIZE_WARP, BlockSize, 2u, Properties::OP_GROUP_SIZE_WARP / BlockSize / 2u>{};
-
-            // template<uint32_t GroupSize, uint32_t BlockSize, uint32_t DupCount, uint32_t Shift>
-            // struct DupTest : OpBase<Properties::OP_ID_SHUFFLE, GroupSize, OP_IMPL_BPERM, OP_CTRL>, detail::amdgcn_duplicate_blocks<GroupSize, BlockSize, DupCount, Shift>{};
-
             // clang-format on
         } // namespace Ops
 
