@@ -39,11 +39,11 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE void
-        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout>& frag,
-                              const DataT*                                                  data,
-                              uint32_t                                                      ldm,
+        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT>& frag,
+                              const DataT*                                                   data,
+                              uint32_t                                                       ldm,
                               uint32_t waveIndex,
                               uint32_t waveCount,
                               uint32_t splitCount)
@@ -57,25 +57,25 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE inline void
-        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout>& frag,
-                              const DataT*                                                  data,
-                              uint32_t                                                      ldm,
+        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT>& frag,
+                              const DataT*                                                   data,
+                              uint32_t                                                       ldm,
                               uint32_t waveIndex,
                               uint32_t waveCount)
     {
 
-        using FragT  = typename std::decay_t<decltype(frag)>;
+        using FragT  = decay_t<decltype(frag)>;
         using Loader = typename GetCoopIOConfig_t<FragT>::Loader;
 
         // Sanity checks
-        static_assert(!std::is_same<DataLayout, void>::value,
+        static_assert(!is_same<DataLayoutT, void>::value,
                       "Must provide layout information. Either statically assign data layout in "
                       "fragment declaration or use the run-time function overload.");
 
         static_assert(
-            std::is_same<typename FragT::Traits::AccessT, typename Loader::Traits::OutputT>::value,
+            is_same<typename FragT::Traits::AccessT, typename Loader::Traits::OutputT>::value,
             "Fragment access and coop load output types do not match");
 
         // Load and implicit pack
@@ -89,13 +89,13 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE void
-        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout>& frag,
-                              const DataT*                                                  data,
-                              uint32_t                                                      ldm)
+        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT>& frag,
+                              const DataT*                                                   data,
+                              uint32_t                                                       ldm)
     {
-        using FragT       = typename std::decay_t<decltype(frag)>;
+        using FragT       = decay_t<decltype(frag)>;
         using MappingUtil = GetMappingUtil_t<FragT>;
 
         // Default: all waves participate in 'row major' order
@@ -107,6 +107,7 @@ namespace rocwmma
         load_matrix_coop_sync(frag, data, ldm, waveIndex, waveCount);
     }
 
+    // @cond
     template <uint32_t WaveCount,
               uint32_t SplitCount,
               typename MatrixT,
@@ -114,16 +115,17 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE void
-        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout>& frag,
-                              const DataT*                                                  data,
-                              uint32_t                                                      ldm,
+        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT>& frag,
+                              const DataT*                                                   data,
+                              uint32_t                                                       ldm,
                               uint32_t waveIndex)
     {
         // SplitCount is unused
         load_matrix_coop_sync<WaveCount>(frag, data, ldm, waveIndex);
     }
+    // @endcond
 
     template <uint32_t WaveCount,
               typename MatrixT,
@@ -131,23 +133,23 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE void
-        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout>& frag,
-                              const DataT*                                                  data,
-                              uint32_t                                                      ldm,
+        load_matrix_coop_sync(fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT>& frag,
+                              const DataT*                                                   data,
+                              uint32_t                                                       ldm,
                               uint32_t waveIndex)
     {
-        using FragT  = typename std::decay_t<decltype(frag)>;
+        using FragT  = decay_t<decltype(frag)>;
         using Loader = typename GetCoopIOConfig_t<FragT, WaveCount>::Loader;
 
         // Sanity checks
-        static_assert(!std::is_same<DataLayout, void>::value,
+        static_assert(!is_same<DataLayoutT, void>::value,
                       "Must provide layout information. Either statically assign data layout in "
                       "fragment declaration or use the run-time function overload.");
 
         static_assert(
-            std::is_same<typename FragT::Traits::AccessT, typename Loader::Traits::OutputT>::value,
+            is_same<typename FragT::Traits::AccessT, typename Loader::Traits::OutputT>::value,
             "Fragment access and coop load output types do not match");
 
         // Load and implicit pack
@@ -161,14 +163,14 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE void store_matrix_coop_sync(
-        DataT*                                                              data,
-        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout> const& frag,
-        uint32_t                                                            ldm,
-        uint32_t                                                            waveIndex,
-        uint32_t                                                            waveCount,
-        uint32_t                                                            splitCount)
+        DataT*                                                               data,
+        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT> const& frag,
+        uint32_t                                                             ldm,
+        uint32_t                                                             waveIndex,
+        uint32_t                                                             waveCount,
+        uint32_t                                                             splitCount)
     {
         // splitCount unused
         store_matrix_coop_sync(data, frag, ldm, waveIndex, waveCount);
@@ -179,24 +181,24 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE void store_matrix_coop_sync(
-        DataT*                                                              data,
-        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout> const& frag,
-        uint32_t                                                            ldm,
-        uint32_t                                                            waveIndex,
-        uint32_t                                                            waveCount)
+        DataT*                                                               data,
+        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT> const& frag,
+        uint32_t                                                             ldm,
+        uint32_t                                                             waveIndex,
+        uint32_t                                                             waveCount)
     {
-        using FragT  = typename std::decay_t<decltype(frag)>;
+        using FragT  = decay_t<decltype(frag)>;
         using Storer = typename GetCoopIOConfig_t<FragT>::Storer;
 
         // Sanity checks
-        static_assert(!std::is_same<DataLayout, void>::value,
+        static_assert(!is_same<DataLayoutT, void>::value,
                       "Must provide data layout. Either statically assign data layout in "
                       "fragment declaration or use the run-time function overload.");
 
         static_assert(
-            std::is_same<typename FragT::Traits::AccessT, typename Storer::Traits::InputT>::value,
+            is_same<typename FragT::Traits::AccessT, typename Storer::Traits::InputT>::value,
             "Fragment access and coop store input types do not match");
 
         // Implicit unpack and store
@@ -210,13 +212,13 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE void store_matrix_coop_sync(
-        DataT*                                                              data,
-        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout> const& frag,
-        uint32_t                                                            ldm)
+        DataT*                                                               data,
+        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT> const& frag,
+        uint32_t                                                             ldm)
     {
-        using FragT       = typename std::decay<decltype(frag)>::type;
+        using FragT       = decay_t<decltype(frag)>;
         using MappingUtil = GetMappingUtil_t<FragT>;
 
         // Default: all waves participate in 'row major' order
@@ -228,6 +230,7 @@ namespace rocwmma
         store_matrix_coop_sync(data, frag, ldm, waveIndex, waveCount);
     }
 
+    // @cond
     template <uint32_t WaveCount,
               uint32_t SplitCount,
               typename MatrixT,
@@ -235,18 +238,19 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE void store_matrix_coop_sync(
-        DataT*                                                              data,
-        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout> const& frag,
-        uint32_t                                                            ldm,
-        uint32_t                                                            waveIndex)
+        DataT*                                                               data,
+        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT> const& frag,
+        uint32_t                                                             ldm,
+        uint32_t                                                             waveIndex)
     {
         // Implicit unpack and store
         // Note: the frag is only be partially filled with useful data.
         // Layout and thread locality is not guaranteed.
         store_matrix_coop_sync<WaveCount>(data, frag, ldm, waveIndex);
     }
+    // @endcond
 
     template <uint32_t WaveCount,
               typename MatrixT,
@@ -254,24 +258,24 @@ namespace rocwmma
               uint32_t BlockN,
               uint32_t BlockK,
               typename DataT,
-              typename DataLayout>
+              typename DataLayoutT>
     ROCWMMA_DEVICE void store_matrix_coop_sync(
-        DataT*                                                              data,
-        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayout> const& frag,
-        uint32_t                                                            ldm,
-        uint32_t                                                            waveIndex)
+        DataT*                                                               data,
+        fragment<MatrixT, BlockM, BlockN, BlockK, DataT, DataLayoutT> const& frag,
+        uint32_t                                                             ldm,
+        uint32_t                                                             waveIndex)
     {
 
-        using FragT  = typename std::decay_t<decltype(frag)>;
+        using FragT  = decay_t<decltype(frag)>;
         using Storer = typename GetCoopIOConfig_t<FragT, WaveCount>::Storer;
 
         // Sanity checks
-        static_assert(!std::is_same<DataLayout, void>::value,
+        static_assert(!is_same<DataLayoutT, void>::value,
                       "Must provide data layout. Either statically assign data layout in "
                       "fragment declaration or use the run-time function overload.");
 
         static_assert(
-            std::is_same<typename FragT::Traits::AccessT, typename Storer::Traits::InputT>::value,
+            is_same<typename FragT::Traits::AccessT, typename Storer::Traits::InputT>::value,
             "Fragment access and coop stor input types do not match");
 
         // Implicit unpack and store
