@@ -39,6 +39,8 @@
 
 #include <rocwmma/internal/types.hpp>
 
+#include "test_config.hpp"
+
 #include "device/common.hpp"
 
 #ifndef CHECK_HIP_ERROR
@@ -55,7 +57,7 @@
     }
 #endif
 
-#ifdef ROCWMMA_BENCHMARK_TESTS
+#if ROCWMMA_BENCHMARK_TESTS
 #ifndef CHECK_RSMI_ERROR
 #define CHECK_RSMI_ERROR(expression, smiErrorFlag)                                               \
     if(auto status = (expression); status != RSMI_STATUS_SUCCESS)                                \
@@ -66,7 +68,7 @@
         smiErrorFlag = true;                                                                     \
     }
 #endif
-#endif
+#endif // ROCWMMA_BENCHMARK_TESTS
 
 namespace rocwmma
 {
@@ -100,6 +102,16 @@ namespace rocwmma
         };
 
 #if !defined(ROCBLAS_DATA_TYPE_FLOAT8)
+        template <>
+        struct rocblas_supported<float8_fnuz_t, float32_t, float32_t> : std::false_type
+        {
+        };
+
+        template <>
+        struct rocblas_supported<bfloat8_fnuz_t, float32_t, float32_t> : std::false_type
+        {
+        };
+#endif
         template <>
         struct rocblas_supported<float8_t, float32_t, float32_t> : std::false_type
         {
@@ -109,7 +121,6 @@ namespace rocwmma
         struct rocblas_supported<bfloat8_t, float32_t, float32_t> : std::false_type
         {
         };
-#endif
 
     } // namespace quirks
 
