@@ -49,20 +49,45 @@ namespace rocwmma
 
         // Native int8
         using TestTypesI8 = std::tuple<
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
             std::tuple<int8_t, int8_t, int32_t>,
 #endif // ROCWMMA_EXTENDED_TESTS
             std::tuple<int8_t, int32_t, int32_t>>;
 
         // float8
-        using TestTypesF8 = std::tuple<std::tuple<float8_t, float32_t, float32_t>>;
+        using TestTypesF8 = std::tuple<
+#if ROCWMMA_FP8
+        std::tuple<float8_t, float32_t, float32_t>
+#endif // ROCWMMA_FP8
 
-        // bfloat8
-        using TestTypesBF8 = std::tuple<std::tuple<bfloat8_t, float32_t, float32_t>>;
+// Only host will ever include both
+#if ROCWMMA_FP8 && ROCWMMA_FP8_FNUZ
+        ,
+#endif // ROCWMMA_FP8 && ROCWMMA_FP8_FNUZ
+
+#if ROCWMMA_FP8_FNUZ
+         std::tuple<float8_fnuz_t, float32_t, float32_t>
+#endif // ROCWMMA_FP8_FNUZ
+        >;
+
+        using TestTypesBF8 = std::tuple<
+#if ROCWMMA_FP8
+        std::tuple<bfloat8_t, float32_t, float32_t>
+#endif // ROCWMMA_FP8
+
+// Only host will ever include both
+#if ROCWMMA_FP8 && ROCWMMA_FP8_FNUZ
+        ,
+#endif // ROCWMMA_FP8 && ROCWMMA_FP8_FNUZ
+
+#if ROCWMMA_FP8_FNUZ
+         std::tuple<bfloat8_fnuz_t, float32_t, float32_t>
+#endif // ROCWMMA_FP8_FNUZ
+        >;
 
         // Non-native bfloat16_t
         using TestTypesBF16 = std::tuple<
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
             std::tuple<bfloat16_t, bfloat16_t, bfloat16_t>,
             std::tuple<bfloat16_t, bfloat16_t, float32_t>,
 #endif // ROCWMMA_EXTENDED_TESTS
@@ -70,7 +95,7 @@ namespace rocwmma
 
         // Native f16
         using TestTypesF16 = std::tuple<
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
             std::tuple<float16_t, float16_t, float16_t>,
             std::tuple<float16_t, float16_t, float32_t>,
 #endif // ROCWMMA_EXTENDED_TESTS
@@ -79,7 +104,7 @@ namespace rocwmma
 #if !ROCWMMA_TESTS_NO_HALF
         // Non-native hfloat16_t (i.e. __half)
         using TestTypesH16 = std::tuple<
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
             std::tuple<hfloat16_t, hfloat16_t, hfloat16_t>,
             std::tuple<hfloat16_t, hfloat16_t, float32_t>,
 #endif // ROCWMMA_EXTENDED_TESTS
@@ -140,28 +165,28 @@ namespace rocwmma
         ///
 
         using TestLayoutsNN =
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
             typename CombineOne<std::tuple<col_major, col_major>, TestDataLayouts>::Result;
 #else
             std::tuple<std::tuple<col_major, col_major, col_major>>;
 #endif // ROCWMMA_EXTENDED_TESTS
 
         using TestLayoutsNT =
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
             typename CombineOne<std::tuple<col_major, row_major>, TestDataLayouts>::Result;
 #else
             std::tuple<std::tuple<col_major, row_major, col_major>>;
 #endif // ROCWMMA_EXTENDED_TESTS
 
         using TestLayoutsTN =
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
             typename CombineOne<std::tuple<row_major, col_major>, TestDataLayouts>::Result;
 #else
             std::tuple<std::tuple<row_major, col_major, col_major>>;
 #endif // ROCWMMA_EXTENDED_TESTS
 
         using TestLayoutsTT =
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
             typename CombineOne<std::tuple<row_major, row_major>, TestDataLayouts>::Result;
 #else
             std::tuple<std::tuple<row_major, row_major, col_major>>;
@@ -177,7 +202,7 @@ namespace rocwmma
 
         // Aggregate combinations BlockK <= 32
         using TestBlockSizes16x16SmallBlockK = std::tuple<std::tuple<I<16>, I<16>, I<16>>
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
                                                           ,
                                                           std::tuple<I<16>, I<16>, I<32>>
 #endif // ROCWMMA_EXTENDED_TESTS
@@ -186,7 +211,7 @@ namespace rocwmma
         // Aggregate combinations BlockK <= 64
         using TestBlockSizes16x16MediumBlockK = std::tuple<std::tuple<I<16>, I<16>, I<16>>,
                                                            std::tuple<I<16>, I<16>, I<32>>
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
                                                            ,
                                                            std::tuple<I<16>, I<16>, I<64>>
 #endif // ROCWMMA_EXTENDED_TESTS
@@ -196,7 +221,7 @@ namespace rocwmma
         using TestBlockSizes16x16LargeBlockK = std::tuple<std::tuple<I<16>, I<16>, I<16>>,
                                                           std::tuple<I<16>, I<16>, I<32>>,
                                                           std::tuple<I<16>, I<16>, I<64>>
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
                                                           ,
                                                           std::tuple<I<16>, I<16>, I<128>>
 #endif // ROCWMMA_EXTENDED_TESTS
@@ -206,7 +231,7 @@ namespace rocwmma
         using TestBlockSizes16x16HugeBlockK = std::tuple<std::tuple<I<16>, I<16>, I<16>>,
                                                          std::tuple<I<16>, I<16>, I<32>>,
                                                          std::tuple<I<16>, I<16>, I<64>>
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
                                                          ,
                                                          std::tuple<I<16>, I<16>, I<128>>,
                                                          std::tuple<I<16>, I<16>, I<256>>
@@ -217,7 +242,7 @@ namespace rocwmma
 
         // Aggregate combinations BlockK <= 16
         using TestBlockSizes32x32SmallBlockK = std::tuple<std::tuple<I<32>, I<32>, I<8>>
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
                                                           ,
                                                           std::tuple<I<32>, I<32>, I<16>>
 #endif // ROCWMMA_EXTENDED_TESTS
@@ -226,7 +251,7 @@ namespace rocwmma
         // Aggregate combinations BlockK <= 32
         using TestBlockSizes32x32MediumBlockK = std::tuple<std::tuple<I<32>, I<32>, I<8>>,
                                                            std::tuple<I<32>, I<32>, I<16>>
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
                                                            ,
                                                            std::tuple<I<32>, I<32>, I<32>>
 #endif // ROCWMMA_EXTENDED_TESTS
@@ -236,7 +261,7 @@ namespace rocwmma
         using TestBlockSizes32x32LargeBlockK = std::tuple<std::tuple<I<32>, I<32>, I<8>>,
                                                           std::tuple<I<32>, I<32>, I<16>>,
                                                           std::tuple<I<32>, I<32>, I<32>>
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
                                                           ,
                                                           std::tuple<I<32>, I<32>, I<64>>
 #endif // ROCWMMA_EXTENDED_TESTS
@@ -246,7 +271,7 @@ namespace rocwmma
         using TestBlockSizes32x32HugeBlockK = std::tuple<std::tuple<I<32>, I<32>, I<8>>,
                                                          std::tuple<I<32>, I<32>, I<16>>,
                                                          std::tuple<I<32>, I<32>, I<32>>
-#if defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_EXTENDED_TESTS
                                                          ,
                                                          std::tuple<I<32>, I<32>, I<64>>,
                                                          std::tuple<I<32>, I<32>, I<128>>
@@ -298,10 +323,10 @@ namespace rocwmma
             {
                 // clang-format off
                 // Don't benchmark wg less than 4 waves by default
-#if defined(ROCWMMA_VALIDATION_TESTS) || defined(ROCWMMA_EXTENDED_TESTS)
+#if ROCWMMA_VALIDATION_TESTS || ROCWMMA_EXTENDED_TESTS
                 {warpSize, 1}, // 1 wave
                 {warpSize, 2}, {warpSize * 2, 1}, // 2 wave
-#endif // ROCWMMA_VALIDATION_TESTS
+#endif // ROCWMMA_VALIDATION_TESTS || ROCWMMA_EXTENDED_TESTS
                 {warpSize, 4}, {warpSize * 2, 2}, // 4 wave
                 {warpSize * 4, 1}  // 4 wave
                 // clang-format on
@@ -310,7 +335,6 @@ namespace rocwmma
 
         static inline std::vector<ProblemSizeT> problemSizes()
         {
-
             return
             {
                 // clang-format off
@@ -323,7 +347,7 @@ namespace rocwmma
                 {512, 512, 512},
                 // Skip validation on larger sizes
                 // due to very slow.
-#if !defined(ROCWMMA_VALIDATION_TESTS)
+#if !ROCWMMA_VALIDATION_TESTS
                 {1024, 1024, 1024},
                 {2048, 2048, 2048},
                 {2560, 2560, 2560},
@@ -331,7 +355,7 @@ namespace rocwmma
                 {3584, 3584, 3584},
                 {4096, 4096, 4096},
                 {5120, 5120, 5120},
-#ifdef ROCWMMA_EXTENDED_TESTS
+#if ROCWMMA_EXTENDED_TESTS
                 {6144, 6144, 6144},
                 {7168, 7168, 7168},
                 {8192, 8192, 8192},
