@@ -39,7 +39,7 @@ namespace rocwmma
         using Base = UnitTestParams;
 
         // Types: Base IOC + double
-        using Types = typename Base::TestTypes16;
+        using Types = typename Base::TestAllSizeTypes;
 
         using DppOps = std::tuple<DppImpl::Ops::ShiftR16<1>, DppImpl::Ops::ShiftR16<13>>;
 
