@@ -38,81 +38,62 @@ namespace rocwmma
     template <uint32_t BlockM,
               uint32_t BlockN,
               typename DataT,
-              typename DataLayoutT,
-              typename std::enable_if_t<
-                  FragSize_guard<BlockM,
+              typename DataLayoutT>
+    __global__ void RowLayout(uint32_t     m,
+                              uint32_t     n,
+                              DataT const* in,
+                              DataT*       out,
+                              uint32_t     ld,
+                              DataT        param1,
+                              DataT        param2)
+    {
+        if constexpr (FragSize_guard<BlockM,
                                  BlockN,
                                  DataT,
                                  DataLayoutT,
                                  Constants::AMDGCN_WAVE_SIZE,
-                                 Constants::AMDGCN_CURRENT_ARCH_ID>::enable()>* = nullptr>
-    __global__ void RowLayout(uint32_t     m,
-                              uint32_t     n,
-                              DataT const* in,
-                              DataT*       out,
-                              uint32_t     ld,
-                              DataT        param1,
-                              DataT        param2)
-    {
-        enum : uint32_t
+                                 Constants::AMDGCN_CURRENT_ARCH_ID>::enable())
         {
-            BlockHeight = BlockM,
-            BlockWidth  = BlockN,
-
-            BlockDim = BlockN,
-            KDim     = BlockM,
-
-            MaxVectorWidth
-            = detail::MaxVWSelector<matrix_b, BlockM, BlockN, DataT, DataLayoutT>::Result,
-            VectorWidth = std::is_same_v<DataLayoutT, col_major> ? MaxVectorWidth : 1
-        };
-
-        using IOTraits = IOTraits<BlockDim, KDim, DataT, VectorWidth>;
-        using LayoutT  = MatrixLayout::Row<BlockDim, KDim, DataT, DataLayoutT, VectorWidth>;
-        using Mapping  = MappingUtil<BlockHeight, BlockWidth, DataT, DataLayoutT>;
-
-        auto baseOffset  = LayoutT::baseOffset();
-        auto iocount     = IOTraits::IOCount;
-        auto matrixCoord = Mapping::matrixCoord();
-
-        enum : uint32_t
-        {
-            MajorIndex = std::is_same_v<DataLayoutT, row_major> ? 0 : 1,
-            MinorIndex = std::is_same_v<DataLayoutT, row_major> ? 1 : 0
-        };
-
-        for(uint32_t i = 0; i < iocount; ++i)
-        {
-            for(uint32_t j = 0; j < VectorWidth; j++)
+            enum : uint32_t
             {
-                auto index = (get<MajorIndex>(matrixCoord) * ld + get<MinorIndex>(matrixCoord))
-                             + Mapping::dataOffset(baseOffset, ld) + j;
-                out[index] = in[index];
+                BlockHeight = BlockM,
+                BlockWidth  = BlockN,
+
+                BlockDim = BlockN,
+                KDim     = BlockM,
+
+                MaxVectorWidth
+                = detail::MaxVWSelector<matrix_b, BlockM, BlockN, DataT, DataLayoutT>::Result,
+                VectorWidth = std::is_same_v<DataLayoutT, col_major> ? MaxVectorWidth : 1
+            };
+
+            using IOTraits = IOTraits<BlockDim, KDim, DataT, VectorWidth>;
+            using LayoutT  = typename LayoutProfile::Row<BlockDim, KDim, DataT, DataLayoutT, VectorWidth>::MatrixLayout;
+            using Mapping  = MappingUtil<BlockHeight, BlockWidth, DataT, DataLayoutT>;
+
+            auto baseOffset  = LayoutT::baseOffset();
+            auto iocount     = IOTraits::IOCount;
+            auto matrixCoord = Mapping::matrixCoord();
+
+            enum : uint32_t
+            {
+                MajorIndex = std::is_same_v<DataLayoutT, row_major> ? 0 : 1,
+                MinorIndex = std::is_same_v<DataLayoutT, row_major> ? 1 : 0
+            };
+
+            for(uint32_t i = 0; i < iocount; ++i)
+            {
+                for(uint32_t j = 0; j < VectorWidth; j++)
+                {
+                    auto index = (get<MajorIndex>(matrixCoord) * ld + get<MinorIndex>(matrixCoord))
+                                + Mapping::dataOffset(baseOffset, ld) + j;
+                    out[index] = in[index];
+                }
+                baseOffset += LayoutT::incrementalOffset(i);
             }
-            baseOffset += LayoutT::incrementalOffset(i);
         }
     }
-
-    template <uint32_t BlockM,
-              uint32_t BlockN,
-              typename DataT,
-              typename DataLayoutT,
-              typename std::enable_if_t<
-                  !FragSize_guard<BlockM,
-                                  BlockN,
-                                  DataT,
-                                  DataLayoutT,
-                                  Constants::AMDGCN_WAVE_SIZE,
-                                  Constants::AMDGCN_CURRENT_ARCH_ID>::enable()>* = nullptr>
-    __global__ void RowLayout(uint32_t     m,
-                              uint32_t     n,
-                              DataT const* in,
-                              DataT*       out,
-                              uint32_t     ld,
-                              DataT        param1,
-                              DataT        param2)
-    {
-    }
+    
 } // namespace rocwmma
 
 #endif // ROCWMMA_DEVICE_ROW_LAYOUT_HPP
