@@ -63,6 +63,12 @@ namespace rocwmma
     template <uint32_t BlockM, uint32_t BlockN, typename DataT, typename Layout>
     dim3 UnitKernelBase<BlockM, BlockN, DataT, Layout>::gridDim() const
     {
+        if(BlockM * mTBlockX < DeviceInfo::instance()->warpSize())
+        {
+            return dim3(0, 0);
+        }
+
+        // There will be a divide by zero error if `BlockM * mTBlockX < DeviceInfo::instance()->warpSize()`
         return dim3(ceilDiv(mM, BlockM * mTBlockX / DeviceInfo::instance()->warpSize()),
                     ceilDiv(mN, BlockN * mTBlockY));
     }
@@ -80,14 +86,9 @@ namespace rocwmma
     bool UnitKernelBase<BlockM, BlockN, DataT, Layout>::checkDevice() const
     {
         auto deviceArch = DeviceInfo::instance()->getGcnArch();
-        return (
-            deviceArch != DeviceInfo::UNSUPPORTED_ARCH
-            && !(deviceArch == DeviceInfo::GFX908
-                 && (std::is_same<DataT, float64_t>::value || std::is_same<DataT, float8_t>::value
-                     || std::is_same<DataT, bfloat8_t>::value))
-            && !(deviceArch == DeviceInfo::GFX90A
-                 && (std::is_same<DataT, float8_t>::value
-                     || std::is_same<DataT, bfloat8_t>::value)));
+        // Unit tests aren't currently testing mma, as these are done in GEMM tests
+        // All types can be tested here for data movement and traits.
+        return deviceArch != DeviceInfo::UNSUPPORTED_ARCH;
     }
 
     template <uint32_t BlockM, uint32_t BlockN, typename DataT, typename Layout>
@@ -98,6 +99,7 @@ namespace rocwmma
         // Forfeit the run because there is no tail for cleanup of remainders.
         auto tileSize = std::make_pair(BlockM * mTBlockX / DeviceInfo::instance()->warpSize(),
                                        BlockN * mTBlockY);
+
         auto gridDims = gridDim();
         return (gridDims.x * std::get<0>(tileSize) == mM)
                && (gridDims.y * std::get<1>(tileSize) == mN);
