@@ -36,7 +36,7 @@ namespace rocwmma
 {
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool bcastTest()
+    ROCWMMA_DEVICE static inline bool bcastTest()
     {
         bool err = false;
 
@@ -54,7 +54,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorIndexTest()
+    ROCWMMA_DEVICE static inline bool iteratorIndexTest()
     {
         bool err = false;
 
@@ -78,7 +78,7 @@ namespace rocwmma
               uint32_t VecSize,
               uint32_t StrideSize,
               typename std::enable_if<(StrideSize >= 1)>::type* = nullptr>
-    __device__ static inline bool iteratorStrideIndexTest(VecT<DataT, VecSize> const& vec)
+    ROCWMMA_DEVICE static inline bool iteratorStrideIndexTest(VecT<DataT, VecSize> const& vec)
     {
         bool err = false;
 
@@ -105,13 +105,13 @@ namespace rocwmma
               uint32_t VecSize,
               uint32_t StrideSize,
               typename std::enable_if<(StrideSize == 0)>::type* = nullptr>
-    __device__ static inline bool iteratorStrideIndexTest(VecT<DataT, VecSize> const& vec)
+    ROCWMMA_DEVICE static inline bool iteratorStrideIndexTest(VecT<DataT, VecSize> const& vec)
     {
         return false;
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorStrideIndexTest()
+    ROCWMMA_DEVICE static inline bool iteratorStrideIndexTest()
     {
         bool err = false;
 
@@ -131,7 +131,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorValueTest()
+    ROCWMMA_DEVICE static inline bool iteratorValueTest()
     {
         bool err = false;
 
@@ -159,7 +159,7 @@ namespace rocwmma
               uint32_t VecSize,
               uint32_t StrideSize,
               typename std::enable_if<(StrideSize >= 1)>::type* = nullptr>
-    __device__ static inline bool iteratorStrideValueTest(VecT<DataT, VecSize> const& vec)
+    ROCWMMA_DEVICE static inline bool iteratorStrideValueTest(VecT<DataT, VecSize> const& vec)
     {
         bool err = false;
         auto it  = makeVectorIterator<StrideSize>(vec).begin();
@@ -182,13 +182,13 @@ namespace rocwmma
               uint32_t VecSize,
               uint32_t StrideSize,
               typename std::enable_if<(StrideSize == 0)>::type* = nullptr>
-    __device__ static inline bool iteratorStrideValueTest(VecT<DataT, VecSize> const& v)
+    ROCWMMA_DEVICE static inline bool iteratorStrideValueTest(VecT<DataT, VecSize> const& v)
     {
         return false;
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorStrideValueTest()
+    ROCWMMA_DEVICE static inline bool iteratorStrideValueTest()
     {
         bool err = false;
 
@@ -212,7 +212,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorSubVectorTypeTest()
+    ROCWMMA_DEVICE static inline bool iteratorSubVectorTypeTest()
     {
         bool err = false;
 
@@ -227,7 +227,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorBeginTest()
+    ROCWMMA_DEVICE static inline bool iteratorBeginTest()
     {
         bool err = false;
 
@@ -247,7 +247,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorEndTest()
+    ROCWMMA_DEVICE static inline bool iteratorEndTest()
     {
         bool err = false;
 
@@ -267,7 +267,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorItTest()
+    ROCWMMA_DEVICE static inline bool iteratorItTest()
     {
         bool err = false;
 
@@ -297,7 +297,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorPostIncDecTest()
+    ROCWMMA_DEVICE static inline bool iteratorPostIncDecTest()
     {
         bool err = false;
 
@@ -326,7 +326,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorPreIncDecTest()
+    ROCWMMA_DEVICE static inline bool iteratorPreIncDecTest()
     {
         bool err = false;
 
@@ -360,7 +360,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorPlusMinusTest()
+    ROCWMMA_DEVICE static inline bool iteratorPlusMinusTest()
     {
         bool err = false;
 
@@ -401,7 +401,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorPlusMinusEqTest()
+    ROCWMMA_DEVICE static inline bool iteratorPlusMinusEqTest()
     {
         bool err = false;
 
@@ -444,7 +444,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorEqTest()
+    ROCWMMA_DEVICE static inline bool iteratorEqTest()
     {
         bool err = false;
 
@@ -475,7 +475,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __device__ static inline bool iteratorRangeBasedForTest()
+    ROCWMMA_DEVICE static inline bool iteratorRangeBasedForTest()
     {
         bool err = false;
 
@@ -496,7 +496,7 @@ namespace rocwmma
     }
 
     template <typename DataT, uint32_t VecSize>
-    __global__ void vectorIteratorTest(uint32_t     m,
+    ROCWMMA_KERNEL void vectorIteratorTest(uint32_t     m,
                                        uint32_t     n,
                                        DataT const* in,
                                        DataT*       out,
